#include "testApp.h"
#include "ofxCvColorImage.h"


//--------------------------------------------------------------
void testApp::setup(){	
	
	ofSetVerticalSync(true);
	ofSetFrameRate(60);
	
	width = ofGetWidth();
	height = ofGetHeight();
	
	VF.setupField(width, height, ofGetWidth(), ofGetHeight());
	VF.randomizeField(2.0);
	
	drawingStyle = 0;
	bFade = false;
	
	hsv = new unsigned char[width * height * 3];
	rgb = new unsigned char[width * height * 3];
	
	texture.allocate(width, height, GL_RGB);
}


//--------------------------------------------------------------
void testApp::exit(){	
	delete[] rgb, hsv;
}

//--------------------------------------------------------------
void testApp::update(){
	
	for(int yy=0; yy<height; ++yy) {
		for(int xx=0; xx<width; ++xx) {
			
			int pos = yy * width + xx;
			ofxVec2f& val = VF.field[pos];
			
			
			float angle = atan2(val.x, val.y);
			float magnitude = sqrt(val.x * val.x + val.y * val.y);
			
			// hsv
			hsv[pos*3 + 0] = ofMap(angle, -M_PI, M_PI, 0, 255);
			hsv[pos*3 + 1] = 255;
			hsv[pos*3 + 2] = ofMap(magnitude, 0, 10, 0, 255);
			
			
			HSVtoRGB(&hsv[pos*3], &rgb[pos*3]);
			
			//cout << ((unsigned int) pixels[pos]) << ", " << ((unsigned int) pixels[pos+1]) << ", " << ((unsigned int) pixels[pos+2]) << endl;
		}
	}
	
	texture.loadData(rgb, width, height, GL_RGB);
	
	//image.blur(5);
	
	if (bFade == true) VF.fadeField(0.99f);
	
}

void testApp::HSVtoRGB(unsigned char *hsv, unsigned char *rgb)
{
	unsigned char &H = hsv[0];
	unsigned char &S = hsv[1];
	unsigned char &V = hsv[2];
	
	unsigned char &R = rgb[0];
	unsigned char &G = rgb[1];
	unsigned char &B = rgb[2];
	
	
	if( V == 0 )
	{ R = 0; G = 0; B = 0; }   
	else if( S == 0 ) {                                                                   
		R = V;                                                           
		G = V;                                                           
		B = V;                                                           
	}                                                                  
	else {                                                                  
		const double hf = H / 60.0;                                      
		const int    i  = (int) floor( hf );                             
		const double f  = hf - i;                                        
		const double pv  = V * ( 1 - S );                                
		const double qv  = V * ( 1 - S * f );                            
		const double tv  = V * ( 1 - S * ( 1 - f ) );                     
		switch( i )                                                       
		{              
			case 0:                                                         
				R = V;                                                        
				G = tv;                                                       
				B = pv;                                                       
				break;                                                        
			case 1:                                                         
				R = qv;                                                       
				G = V;                                                        
				B = pv;                                                       
				break;                                                        
			case 2:                                                         
				R = pv;                                                       
				G = V;                                                        
				B = tv;                                                       
				break;                                                        
			case 3:                                                         
				R = pv;                                                       
				G = qv;                                                       
				B = V;                                                        
				break;                                                        
			case 4:                                                         
				R = tv;                                                       
				G = pv;                                                       
				B = V;                                                        
				break;                                                        
			case 5:                                                         
				R = V;                                                        
				G = pv;                                                       
				B = qv;                                                       
				break;                                                        
			case 6:                                                         
				R = V;                                                        
				G = tv;                                                       
				B = pv;                                                       
				break;                                                        
			case -1:                                                        
				R = V;                                                        
				G = pv;                                                       
				B = qv;                                                       
				break;                                                        
				
			default:                                                        
				cerr << "i Value error in Pixel conversion, Value is " << i << endl;  
				break;                                                        
		}                                                               
	}   
	
	R *= 255.0F;                                                        
	G *= 255.0F;                                                        
	B *= 255.0F;
	
}

//--------------------------------------------------------------
void testApp::draw(){
	/*
	 ofEnableAlphaBlending();
	 //ofSetColor(0,130,130, 200);
	 //VF.draw();
	 
	 
	 ofSetColor(255, 255, 255);
	 image.draw(0, 0);
	 
	 ofSetColor(0,130,130, 200);
	 ofRect(30,30,300,75);
	 ofSetColor(255,255,255, 230);
	 ofDrawBitmapString("space to clear\nchange drawing mode 'a'\ntoggle fade 'f'", 50, 50);
	 
	 
	 ofSetColor(255,255,130, 230);
	 switch (drawingStyle){
	 case 0: ofDrawBitmapString("drawing mode: inward", 50, 90);
	 break;
	 case 1: ofDrawBitmapString("drawing mode: outward", 50, 90);
	 break;
	 case 2: ofDrawBitmapString("drawing mode: clockwise", 50, 90);
	 break;
	 case 3: ofDrawBitmapString("drawing mode: counter-clockwise", 50, 90); 
	 break;
	 }	
	 */
	//image.draw(100, 100);
	texture.draw(0, 0, width, height);
}

//--------------------------------------------------------------
void testApp::keyPressed  (int key){ 
	
	if (key == ' '){
		VF.clear();
	} else if (key == 'a'){
		drawingStyle ++;
		drawingStyle %= 4;
	} else if (key == 'f'){
		bFade = !bFade;
	}
}

//--------------------------------------------------------------
void testApp::keyReleased  (int key){ 
}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){
}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){
	if (button == 0) {
		switch (drawingStyle){
			case 0: VF.addInwardCircle((float)x, (float)y, 200, 0.3f);
				break;
			case 1: VF.addOutwardCircle((float)x, (float)y, 100, 0.3f);
				break;
			case 2: VF.addClockwiseCircle((float)x, (float)y, 100, 0.3f);
				break;
			case 3: VF.addCounterClockwiseCircle((float)x, (float)y, 100, 0.3f);
				break;
		}		
	} else {
		
		particles.erase(particles.begin());
		particle myParticle;
		myParticle.setInitialCondition(x,y,0,0);
		particles.push_back(myParticle);
		
		
	}
}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){
}

//--------------------------------------------------------------
void testApp::mouseReleased(){
}
